'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = createManager;

var _store = require('./store');

var _renderer = require('./renderer');

var _renderer2 = _interopRequireDefault(_renderer);

var _connect = require('react-redux/lib/components/connect');

var _connect2 = _interopRequireDefault(_connect);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _take = require('lodash/take');

var _take2 = _interopRequireDefault(_take);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var mapStateToProps = function mapStateToProps(state) {
    return {
        items: state.items
    };
};

var ConnectedRenderer = (0, _connect2.default)(mapStateToProps)(_renderer2.default);

function createManager() {
    var store = (0, _store.createStore)();
    var renderer = _react2.default.createElement(ConnectedRenderer, { store: store });

    return {
        _store: store,

        GlobalOptions: {
            cacheTime: 5 * 60 * 1000,
            cacheLimit: 5
        },

        renderer: renderer,

        activatePage: function activatePage(_ref) {
            var id = _ref.id;
            var args = _ref.args;
            var page = _ref.page;
            var _ref$cacheTime = _ref.cacheTime;
            var cacheTime = _ref$cacheTime === undefined ? this.GlobalOptions.cacheTime : _ref$cacheTime;

            var store = this._store;

            // for other active pages (if work correctly, there will be at most one)
            store.getState().items.filter(function (item) {
                return item.isActive && item.id !== id;
            }).forEach(function (item) {
                // if cache time < 0, do not cache this page and remove it just as it is inactivated
                if (item.cacheTime < 0) {
                    store.dispatch(_store.actions.removeItem(item.id));
                } else {
                    // set clear timer (null = forever)
                    var timer = void 0;
                    if (item.cacheTime !== null) {
                        timer = setTimeout(function () {
                            store.dispatch(_store.actions.removeItem(item.id));
                        }, item.cacheTime);
                    }

                    // inactive it
                    store.dispatch(_store.actions.setItem(Object.assign({}, item, {
                        isActive: false,
                        cacheTimer: timer,

                        // record scroll position when it is inactivated to recover for the next time it get activated
                        scrollTop: document.body.scrollTop
                    })));
                }
            });

            // for current page
            var scrollTop = 0;
            {
                var item = (0, _find2.default)(store.getState().items, { id: id });

                // if found cache
                if (item) {
                    // remove the clear timer
                    clearTimeout(item.cacheTimer);

                    // if current page is already active, stick to its current scroll
                    if (item.isActive) {
                        scrollTop = document.body.scrollTop;
                    }
                    // else, it is re-activated, recover its old scroll
                    else {
                            scrollTop = item.scrollTop;
                        }
                }

                // activate it
                store.dispatch(_store.actions.setItem({ id: id, args: args, page: page, lastActivatedAt: new Date(), isActive: true, cacheTime: cacheTime }));
            }

            // remove over limit pages
            store.dispatch(_store.actions.setItems((0, _take2.default)(store.getState().items.slice().sort(function (a, b) {
                return b.lastActivatedAt.getTime() - a.lastActivatedAt.getTime();
            }), this.GlobalOptions.cacheLimit)));

            // recover scroll
            document.body.scrollTop = scrollTop;
            setTimeout(function () {
                return document.body.scrollTop = scrollTop;
            }, 0);
        }
    };
}